@@ -7,7 +7,7 @@ BEGIN {
 }
 
 
-our $VERSION =  '0.03';
+our $VERSION =  '0.04';
 
 =head1 NAME
 
@@ -38,155 +38,146 @@ C<Devel::DebugHooks> - Hooks for perl debugger
 BEGIN {
 	unless( defined $DB::dbg ) {
 		my $level =  0;
-		while( my @frame =  caller($level++) ) {
-			$DB::dbg =  $frame[0]   if $frame[0] =~ /^Devel::/;
+		while( my $frame =  caller($level++) ) {
+			$DB::dbg =  $frame   if $frame =~ /^Devel::/;
 		}
+
+		$DB::dbg //=  __PACKAGE__;
+
 		# ISSUE: We can not make 'main' package as descendant of 'Devel::DebugHooks'
 		# because of broken info from 'caller', so I restrict pkg_names to 'Devel::'
 		# TODO: Ask #p5p about that 'caller' shows strange file:line for (eval)
 		# https://rt.perl.org/Public/Bug/Display.html?id=127083
+	}
+}
 
-		unless( defined $DB::dbg ) {
-			$DB::dbg =  'Devel::DebugHooks::Verbose';
-			@DB::options{ qw/ trace_load trace_subs trace_returns / } = ( 1, 1, 1 );
+
+
+sub pop_frame {
+	#NOTICE: We will fall into infinite loop if something dies inside this sub
+	#because this sub is called when flow run out of scope.
+	#TODO: Put this code into eval block
+
+	my $stack =  DB::state( 'stack' );
+	my $last =  pop @$stack;
+	DB::print_state( "POP  FRAME <<<< ",
+		"  --  $last->{ sub }\@". @$stack ."\n"
+		."    $last->{ file }:$last->{ line }\n\n"
+	) if DB::state( 'ddd' );
+
+	if( @{ DB::state( 'stack' ) } ) {
+		# Restore $DB::single for upper frame
+		DB::state( 'single', DB::state( 'single' ) );
+	} else {
+		# Something nasty happened at &push_frame, because of we are at
+		# &pop_frame already but not "push @{ state( 'stack' ) }" done yet
+		print $DB::OUT "Error happen while &pop_frame. Pay attention to this!\n";
+		$DB::single =  0;
+	}
+}
+
+
+
+{
+	my $handler =  DB::reg( 'pop_frame', 'DebugHooks' );
+	$$handler->{ context } =  $DB::dbg;
+	$$handler->{ code }    =  \&pop_frame;
+}
+
+
+
+
+sub test {
+	1;
+	2;
+}
+
+
+
+sub push_frame {
+	{ # these lines exists for testing purpose
+		no warnings 'void';
+		test();
+		3;
+	}
+
+	shift; #Turf event context
+	my $sub =  shift;
+	DB::print_state( "PUSH FRAME $_[0] >>>>  ", "  --  $sub\n" )   if DB::state( 'ddd' );
+
+	if( $_[0] ne 'G' ) {
+		# http://stackoverflow.com/questions/34595192/how-to-fix-the-dbgoto-frame
+		# WORKAROUND: for broken frame. Here we are trying to be closer to goto call
+		# Most actual info we get when we trace script step-by-step at this case
+		# those vars have sharp last opcode location.
+		if( !DB::state( 'eval' ) ) {
+			#TODO: If $DB::single == 1 we can skip this because cursor is updated at DB::DB
+			my( $p, $f, $l ) =  caller 5;
+			DB::state( 'package', $p );
+			DB::state( 'file',    $f );
+			DB::state( 'line',    $l );
+			DB::print_state( "", sprintf "\n    cursor(PF) => %s, %s, %s\n" ,$p ,$f, $l )   if DB::state( 'ddd' );
+		}
+
+		my $stack =  DB::state( 'stack' );
+		my $frame =  {()
+			# Until we stop at a callee last known cursor position is the caller position
+			,package     =>  $stack->[-1]{ package }
+			,file        =>  $stack->[-1]{ file    }
+			,line        =>  $stack->[-1]{ line    }
+			,single      =>  $stack->[-1]{ single  }
+			,sub         =>  $sub
+			,goto_frames =>  []
+			,type        =>  $_[0]
+		};
+
+
+		DB::emit( 'frame', $frame );
+
+		#TODO: Now we push always. Q: How to skip coresopnding &pop_frame?
+		# Think about this feature: if( $confirm ) {
+		push @{ $stack }, $frame;
+	}
+	else {
+		push @{ DB::state( 'goto_frames' ) },
+			[ DB::state( 'package' ), DB::state( 'file' ), DB::state( 'line' ), $sub, $_[0] ]
+	}
+
+
+	DB::emit( 'call', $sub );
+	1;
+}
+
+
+
+{
+	my $handler =  DB::reg( 'push_frame', 'DebugHooks' );
+	$$handler->{ context } =  $DB::dbg;
+	$$handler->{ code }    =  \&push_frame;
+}
+
+
+
+# NOTICE: We do not inherit DB:: interface, we use it
+sub import {
+	if( $_[0] eq 'Devel::DebugHooks' ) {
+		shift;
+		for my $module ( @_ ) {
+			my( $package, $args ) =  $module =~ m/^([^=]+)=?(.*)$/;
+			$DB::dbg =  "Devel::DebugHooks::$package";
+			$package  =~ s/::/\//;
+			require "Devel/DebugHooks/$package.pm";
+			$DB::dbg->import( split ':', $args );
 		}
 	}
-}
-
-
-
-sub init {
-}
-
-
-
-sub import {
-	DB::import( @_ );
-}
-
-
-
-sub trace_load {
-	my $self =  shift;
-
-	return "Loaded '@_'\n"
-}
-
-
-
-# This sub is called for each DB::DB call when $DB::trace is true
-sub trace_line {
-	print DB::state( 'line' ) ."\n";
-}
-
-
-
-sub bbreak {
-	my $info =  "\n" .' =' x30 .DB::state( 'inDB' ) ."\n";
-
-	$info .=  sprintf "%s:%s    %s"
-		,DB::state( 'file' )
-		,DB::state( 'line' )
-		,DB::source()->[ DB::state( 'line' ) ]
-	;
-
-	return $info;
-}
-
-
-
-sub interact {
-}
-
-
-
-# Q: May this unit pull other units so we will not be able to see loading process?
-# A: No, this will be done at run time. Until that trace_load will be visible
-eval 'require ' .$DB::options{ cmd_processor }; die $@   if $@;
-sub process {
-	BEGIN{ 'strict'->unimport( 'refs' )   if $DB::options{ s } }
-	# TODO: if we set trap on sub that loaded at CT, this will FAIL
-	# move require here
-	&{ $DB::options{ cmd_processor } .'::process' }( @_ );
-}
-
-
-
-# NOTICE: &DB::sub is not called because of DB::namespace
-sub abreak {
-}
-
-
-
-my %frame_name;
-BEGIN {
-	%frame_name =  (
-		G => 'GOTO',
-		D => 'DBGF',
-		C => 'FROM',
-	);
-}
-
-sub trace_subs {
-	my( $self ) =  @_;
-
-	BEGIN{ 'warnings'->unimport( 'uninitialized' )   if $DB::options{ w } }
-
-
-	my $info = '';
-	local $" =  ' -';
-	my( $orig_frame, $last_frame );
-	for my $frame ( DB::frames() ) {
-		$last_frame //=  $frame   if $frame->[0] ne 'D';
-		$orig_frame //=  $frame   if $frame->[0] ne 'D'  &&  $frame->[0] ne 'G';
-
-		$info .=  $frame_name{ $frame->[0] } .": @$frame[2..5]\n";
+	else {
+		DB::import( @_ );
 	}
-
-	my $context = $orig_frame->[7] ? 'list'
-			: defined $orig_frame->[7] ? 'scalar' : 'void';
-
-	$" =  ', ';
-	my @args =  map { !defined $_ ? '&undef' : $_ } @{ $orig_frame->[1] };
-	$info =
-		"\n" .' =' x15 ."\n"
-		."DEEP: ". @{ DB::state( 'stack' ) } ."\n"
-		."CNTX: $context\n"
-		.$last_frame->[0] ."SUB: " .$last_frame->[5] ."( @args )\n"
-		# print "TEXT: " .DB::location( $DB::sub ) ."\n";
-		# NOTICE: even before function call $DB::sub changes its value to DB::location
-		# A: Because @_ keep the reference to args. So
-		# 1. The reference to $DB::sub is saved into @_
-		# 2. The DB::location is called
-		# 3. The value of $DB::sub is changed to DB::location
-		# 4. my( $sub ) =  @_; # Here is too late to get the orig value of $DB::sub
-		."TEXT: " .DB::location( $last_frame->[5] ) ."\n\n"
-		.$info;
-
-	$info .=  ' =' x15 ."\n";
-
-	return $info;
+	# shift->SUPER::import( @_ );
 }
 
 
 
-sub trace_returns {
-	my $self =  shift;
-
-	my $info;
-	$info =  $DB::options{ trace_subs } ? '' : "\n" .' =' x15 ."\n";
-	# FIX: uninitializind value while 'n'
-	# A: Can not reproduce...
-	$info .= join '->', map { $_->[3] } @{ DB::state( 'goto_frames' ) };
-	$info .= " RETURNS:\n";
-
-	$info .=  @_ ?
-		'  ' .join "\n  ", map { defined $_ ? $_ : '&undef' } @_:
-		'>>NOTHING<<';
-
-	return $info ."\n" .' =' x15 ."\n";
-}
-
 package
 	x;
 
@@ -209,52 +200,35 @@ sub X {
 
 
 
-package    # hide the package from the PAUSE indexer
-	Devel::DebugHooks::Verbose;
+package DB::Error;
+use overload bool => sub {1}, '""' => sub { shift->error }, fallback => 1;
 
-our @ISA;
-
-BEGIN {
-	push @ISA, 'Devel::DebugHooks';
-}
-
-sub trace_load {
-	my $self =  shift;
-
-	print $DB::OUT $self->SUPER::trace_load( @_ );
-}
-
-sub trace_subs {
-	my $self =  shift;
-
-	print $DB::OUT $self->SUPER::trace_subs( @_ );
-}
-
-sub trace_returns {
-	my $self =  shift;
-
-	print $DB::OUT $self->SUPER::trace_returns( @_ );
-}
-
-sub bbreak {
-	my $self =  shift;
-
-	print $DB::OUT $self->SUPER::bbreak( @_ );
+sub error {
+	return shift->{ error };
 }
 
 
-package    # hide the package from the PAUSE indexer
-	DB;
+package DB;
 
 # In theory this may break user's code because this usage cause dependencies are loaded
 # in different order under debugger
 use Scope::Cleanup qw/ establish_cleanup /;
 use Sub::Metadata qw/ mutate_sub_is_debuggable /;
+use List::Util;
+
+BEGIN {
+	# https://metacpan.org/pod/release/PEVANS/Scalar-List-Utils-1.27/lib/List/Util.pm#SUGGESTED-ADDITIONS
+	# Perl <=5.18 have List::Utils 1.27 which have not next functions:
+	$List::Util::VERSION <= 1.27  &&  eval '
+		sub List::Util::any(&@) { my $sub =  shift;	$sub->() && return 1 for @_; 0 }
+		sub List::Util::all(&@) { my $sub =  shift; $sub->() || return 0 for @_; 1 }
+	';
+}
 
 
 
 ## Utility subs
-sub _all_frames {
+sub orig_frames {
 	BEGIN{ 'warnings'->unimport( 'uninitialized' )   if $DB::options{ w } }
 	my( $count, $warn ) =  @_;
 	$count //=  -1; # infinite
@@ -262,7 +236,7 @@ sub _all_frames {
 	my $lvl =  0;
 	# $x  &&  $y = 3 in this case '=' op precedence should be higher then &&
 	while( $count--  &&  (my @frame =  caller( $lvl++ )) ) {
-		$_ =  "ORIG: @frame[0..3,5]\n";
+		$_ =  "@frame[0..3,5]\n";
 		print $DB::OUT $_   unless $warn;
 		warn $_             if $warn;
 	}
@@ -290,9 +264,10 @@ sub print_state {
 
 	my( undef, $f, $l ) =  caller;
 	print $DB::OUT
-		$before ."$f:$l: "
+		$before
 		."DB::state: l:" .@$DB::state ." d:" .(DB::state('inDB')//0)
 		." s:$DB::single t:$DB::trace"
+		."  $f:$l"
 		.($after // "\n")
 	;
 }
@@ -316,7 +291,7 @@ sub log_access {
 		# Also this makes life happy when we compare diff for two debugger output
 		$old_value =  ref $old_value ? ref $old_value : $old_value;
 		$new_value =  ref $new_value ? ref $new_value : $new_value;
-		print $DB::OUT " $context::$name: $old_value$new_value\n";
+		print $DB::OUT " ${context}::$name: $old_value$new_value\n";
 	}
 
 
@@ -371,6 +346,8 @@ sub frm_vrbl {
 
 mutate_sub_is_debuggable( \&state, 0 );
 sub state {
+	return   unless $DB::state  &&  @$DB::state;
+
 	my( $name, $value ) =  @_;
 
 	# Do not debug access into 'ddd' flag
@@ -382,7 +359,7 @@ sub state {
 	# Show full stack only when verbose mode ON and only before changes
 	# or before implicit changes: when debugger command take whole 'stack'
 	# and manipulate data directly
-	if( $debug  &&  $debug >= 3  && ( @_ >= 2 || $name eq 'stack' ) ) {
+	if( $debug  &&  (($debug >= 3  &&  @_ >= 2)  ||  $name eq 'stack' ) ) {
 		print_state "\n    ";
 
 		for( @$DB::state ) {
@@ -417,7 +394,7 @@ sub state {
 	if( $debug && ( @_ >= 2 || $debug >= 2 ) ) {
 		my($file, $line) =  (caller 0)[1,2];
 		$file =~ s'.*?([^/]+)$'$1'e;
-		print $DB::OUT -$level ." $file:$line: ";
+			print $DB::OUT "    " .-$level ." $file:$line: ";
 	}
 
 	unless( $instance ) {
@@ -429,6 +406,11 @@ sub state {
 
 
 
+	if( $name eq 'instance' ) {
+		print $DB::OUT " INT::instance\n"   if $debug && ( @_ >= 2 || $debug >= 2 );
+		return $instance;
+	}
+
 	$name =  '*'   unless exists $DB::variables->{ $name };
 	return $DB::variables->{ $name }({()
 			,debug    =>  $debug
@@ -447,16 +429,28 @@ sub _ddd {
 
 
 sub new {
+	my $ddd =  DB::state( 'ddd' );
+
 	# NOTICE: After creating new debugger instance we are in debugger yet
 	# So we set { inDB } flag. It allows us safely initialize new debugger
-	# instance through &DB::state ( see &DB::state )
-	push @$DB::state, { inDB => 1, stack =>  [ {()
+	# instance through &DB::state ( see &DB::state ). We do not do that directly
+	# to spy which state and how it is changed when { ddd } is turned on
+	my $dbg_instance =  bless { inDB => 1, stack =>  [ {()
 		,goto_frames =>  []
 		# ,type => 'D'
-	} ], @_ };
+	} ], @_ }, $DB::dbg;
+	push @$DB::state, $dbg_instance;
 
-	print $DB::OUT "IN  DEBUGGER  >>>>>>>>>>>>>>>>>>>>>> \n"
-		if _ddd;
+	# New debugger instance should have same { ddd } flag
+	# DB::state( 'ddd', $ddd )   if defined $ddd;
+
+	if( @$DB::state > 1 ) {
+		my @events =  grep{ /^on_/ } keys %{ $DB::state->[-2] };
+		@{ $DB::state->[-1] }{ @events } =  @{ $DB::state->[-2] }{ @events };
+	}
+
+	print $DB::OUT "\nIN DEBUGGER  >>>>>>>>>>>>>>>>>>>>>>\n\n"
+		if DB::state( 'ddd' );
 
 	my $self;
 }
@@ -466,8 +460,9 @@ sub new {
 sub DESTROY {
 	# Clear { dd } flag to prevent debugging for next command
 	DB::state( 'dd', undef );
+	DB::state( 'debug', undef );
 
-	print $DB::OUT "OUT DEBUGGER  <<<<<<<<<<<<<<<<<<<<<< \n"
+	print $DB::OUT "\nOUT DEBUGGER  <<<<<<<<<<<<<<<<<<<<<<\n\n"
 		if _ddd;
 	pop @$DB::state;
 	# NOTICE: previous frame is always have { inDB } flag on because there is
@@ -493,13 +488,13 @@ sub DESTROY {
 # to debug it from first OP. We can disable this feature through NonStop option.
 
 our $dbg;            # debugger object/class
-our $package;        # current package
-our $file;           # current file
-our $line;           # current line number
-our @goto_frames;    # save sequence of places where nested gotos are called
+# our $package;        # current package
+# our $file;           # current file
+# our $line;           # current line number
+# our @goto_frames;    # save sequence of places where nested gotos are called
 our $commands;       # hash of commands to interact user with debugger
-our @stack;          # array of hashes that keeps aliases of DB::'s ours for current frame
-					 # This allows us to spy the DB::'s values for a given frame
+# our @stack;          # array of hashes that keeps aliases of DB::'s ours for current frame
+                       # This allows us to spy the DB::'s values for a given frame
 # our $ddlvl;          # Level of debugger debugging <= @$DB::state
 # our $dbg_call;       # keep silent at DB::sub/lsub while do external call from DB::*
 # our $inDB;           # Flag which shows we are currently in debugger
@@ -509,8 +504,6 @@ our $IN;
 our $OUT;
 our %options;
 our $interaction;    # True if we interact with dbg client
-our %stop_in_sub;    # In this hash the key is a sub name we want to stop on
-					 # maintained at 'trace_subs'
 our $variables;      # Hash which defines behaviour for values available through &DB::state
 	# There three types of variables:
 	# Debugger internal variables -- global values from DB:: package
@@ -541,7 +534,6 @@ BEGIN {
 	$OUT                       //= \*STDOUT;
 
 	#FIX: Where apply 'ddd' from command line?
-	#TODO: Describe why we do not use { inDB } flag here
 	DB::new;
 	DB::state( single =>  $DB::single );
 
@@ -555,13 +547,6 @@ BEGIN {
 	# TODO: camelize options. Q: Why?
 	$options{ frames }         //=  -1;        # compile time & runtime option
 	$options{ dbg_frames }     //=  0;         # compile time & runtime option
-	# The differece when we set option at compile time, we see module loadings
-	# and compilation order whereas setting up it at run time we lack that info
-	$options{ trace_load }     //=  0;         # compile time option
-	$options{ trace_subs }     //=  0;         # compile time & runtime option
-	$options{ trace_returns }  //=  0;
-
-	$options{ cmd_processor }  //=  'Devel::DebugHooks::CmdProcessor';
 
 	#options{ save_path } # TODO: save code path for displaying by graphviz
 	$DB::postponed{ 'DB::DB' } =  1;
@@ -645,6 +630,13 @@ BEGIN { # Initialization goes here
 	{
 		BEGIN{ 'strict'->unimport( 'refs' )   if $options{ s } }
 
+		sub dd {
+			eval "use Data::Dump";
+			Data::Dump::pp( @_ );
+		}
+
+
+
 		# Returns TRUE if $filename was compiled/evaled
 		# The file is evaled if it looks like (eval 34)
 		# But this may be changed by #file:line. See ??? for info
@@ -763,20 +755,16 @@ BEGIN { # Initialization goes here
 		DB::state( 'inDB', undef );
 
 
+		# Read BEWARE at DebugHooks.pod about localization of globals
 		local $^D;
-		# FIX: when we eval user's sub the main script frame is changed
-
-		# BEWARE: We should local'ize every global variable the debugger make change
-		# If we forgot that we will hurt user's context.
-		# Here we should localize only those which values are changed implicitly
-		# or indirectly: exceptions, signals...
-		# In a word those circumstances your code can not control
-		# local $_ =  $DB::context[4]; (See commit:035e182e4f )
-
+		local $_ =  $DB::context[5];
 		local @_ =  @{ $DB::context[0] };
 		eval "$usercontext; package $package;\n#line 1\n$expr";
 		#NOTICE: perl implicitly add semicolon at the end of expression
 		#HOWTO reproduce. Run command: X::X;1+2
+		#
+		# print $DB::OUT "Error occur while evaluating: $@"   if $@
+		# But if we do this we return wrong value
 	}
 
 
@@ -820,7 +808,7 @@ BEGIN { # Initialization goes here
 		}
 
 
-		_all_frames()   if $options{ _all_frames };
+		orig_frames()   if $options{ orig_frames };
 
 
 		# For uninitialized values in frames
@@ -914,50 +902,38 @@ BEGIN { # Initialization goes here
 	# TODO: implement $DB::options{ trace_internals }
 	sub mcall {
 		my $method  =  shift;
-		my $context =  shift;
-		my $sub =  $context->can( $method );
+		my $context =  DB::state( 'instance' );
 
-		my $dd =  DB::state( 'dd' ) // 0;
 		print "mcall ${context}->$method\n"
-			if DB::state( 'ddd' )  ||  $dd >= 2;
+			if DB::state( 'ddd' );
+
+		my $sub =  $context->can( $method );
 		scall( $sub, $context, @_ );
 	}
 
 
 
 	sub scall {
-		# When we start debugger debugging with verbose output
-		# { dd } >= 2 the user frame may have not { ddd } but
-		# we should not lose any output info
-		# until we setup initial state for new debugger instance
-		# See "Create new debugger's state instance" below
-		my $dd =  DB::state( 'dd' ) // 0;
-		my $ddd =  DB::state( 'ddd' )  #||  { dd } >= 2
-			# HACK: Always return level of debugging output
-			||  $dd && ($dd -1);
-
-		# TODO: implement debugger debugging
-		# local $^D |= (1<<30);
-		my( $from, $f, $l, $sub );
+		my $sub =  sub_name( $_[0] ) || $_[0];
+		my $ddd =  DB::state( 'ddd' );
+		my( $from, $f, $l );
 		if( $ddd ) {
 			my $lvl =  0;
 			if( (caller 1)[3] eq 'DB::mcall' ) {
 				$lvl++;
-				$sub =  "$DB::args[1]::$DB::args[0]";
-			}
-			else {
-				$sub =  $DB::_sub; #FIX: remove global
 			}
 
+			{ local $" =  ', '; $sub .=  "( @_[ 1..$#_ ] )"; }
+
 			($f, $l) =  (caller $lvl)[1,2];
 			$f =~ s".*?([^/]+)$"$1";
 			$from =  (caller $lvl+1)[3];
 
-			print $DB::OUT ">> scall from $from($f:$l) --> $sub\n"
+			print $DB::OUT ">> scall from $from($f:$l) --> $sub\n";
 		}
 
-		die "You can make debugger call only from debugger"
-			unless DB::state( 'inDB' );
+		# die "You can make debugger call only from debugger"
+		# 	unless DB::state( 'inDB' );
 
 
 		# FIX: http://perldoc.perl.org/perldebguts.html#Debugger-Internals
@@ -972,18 +948,32 @@ BEGIN { # Initialization goes here
 		# local *DB::sub =  *DB::sub; *DB::sub =  $stub;
 		# Another: the { inDB } flag
 
+		#IT: Call other debugger commands from current command
+		my $old_cmd =  DB::state( 'cmd' );
+
 		# Manual localization
 		my $scall_cleanup =  sub {
-			print $DB::OUT "Debugger command DONE\n"   if $ddd;
-
-			DB::state( 'cmd', undef );
+			print_state "Debugger command DONE: $sub  "   if $ddd;
 
 			# NOTICE: Because  we are in debugger here we should setup { inDB }
 			# flag but we are leaving debugger and interesting at user's context
-			# $DB::single =  DB::state( 'single' );
-			DB::state( 'single', DB::state( 'single' ) );
 
-			DB::DESTROY   if DB::state( 'dd' );
+			DB::state( 'cmd', $old_cmd ); DB::state( 'single', 0, 1 )   if $old_cmd;
+			# $DB::single =  DB::state( 'single' );
+			DB::state( 'single', DB::state( 'single' ) ) unless $old_cmd;
+
+			my $dd;
+			DB::DESTROY   if ($dd =  DB::state( 'dd' ))  &&  $sub =~ /$dd/;
+
+			# Enable debugging after current command is finished
+			if( my $debug =  DB::state( 'debug' ) ) {
+				DB::state( 'debug', undef );
+				my( $verbose, $sub ) =  $debug =~ /^(\d+)?(?:@(.*))?$/;
+				# Set or flush debug flags depending on user's input
+				DB::state( 'ddd', $verbose // undef );
+				DB::state( 'dd',  $sub     // undef );
+			}
+
 
 			print $DB::OUT "<< scall back $from($f:$l) <-- $sub\n"   if $ddd;
 		};
@@ -994,14 +984,12 @@ BEGIN { # Initialization goes here
 
 
 		# Create new debugger's state instance
-		if( my $dd =  DB::state( 'dd' ) ) {
+		my $dd;
+		if( ($dd =  DB::state( 'dd' ))  &&  $sub =~ /$dd/ ) {
 			# NOTICE: We should not set debugger states directly when create
 			# new state instance. We will not see changes at debug output
 			# So we use &DB::state after instance initialization
-			DB::new(()
-				# A new debugger instance has its own { ddd } flag
-				,( $dd >= 2 ? (ddd => $dd -1) : () )
-			);
+			DB::new();
 			DB::state( 'single', 1 );
 			DB::state( 'inDB', undef );
 			$^D |=  1<<30;
@@ -1011,7 +999,7 @@ BEGIN { # Initialization goes here
 			DB::state( 'cmd', 1 );
 		}
 
-		print $DB::OUT "Call debugger command\n"   if $ddd;
+		print $DB::OUT "Call debugger command: $sub\n"   if $ddd;
 		return shift->( @_[ 1..$#_ ] );
 
 		# my $method =  shift;
@@ -1023,7 +1011,7 @@ BEGIN { # Initialization goes here
 
 	sub save_context {
 		@DB::context =  ( \@_, (caller 2)[8..10], $@, $_ );
-		print $DB::OUT "\nTRAPPED IN: " .@$DB::state ."\n\n"   if _ddd;
+		print_state "\nTRAPPED IN  ", "\n\n"   if _ddd;
 		DB::state( 'inDB', 1 );
 	}
 
@@ -1034,8 +1022,11 @@ BEGIN { # Initialization goes here
 	mutate_sub_is_debuggable( \&restore_context, 0 );
 	sub restore_context {
 		DB::state( 'inDB', undef );
-		print_state '', "\nTRAPPED OUT: " .@$DB::state ."\n\n"   if _ddd;
+		print_state "\nTRAPPED OUT  ", "\n\n"   if _ddd;
 		$@ =  $DB::context[ 4 ];
+		# WARNING: Do not keep any references to user's data, in other case we postpone
+		# object desctruction process.
+		@DB::context =  (); # TODO: IT
 	}
 } # end of provided DB::API
 
@@ -1055,7 +1046,6 @@ sub import { # NOTE: The import is called at CT yet
 	# NOTICE: it is useless to set breakpoints for not compiled files
 	# TODO: spy module loading and set breakpoints
 	#TODO: Config priority: conf, ENV, cmd_line
-	$DB::dbg->init();
 
 
 	# Parse cmd_line options:
@@ -1086,22 +1076,45 @@ sub import { # NOTE: The import is called at CT yet
 }
 
 
+
+# use Sub::Identify qw/ sub_name /;
+use B qw(svref_2object);
+sub sub_name {
+    return unless ref( my $r = shift );
+    return unless my $cv = svref_2object( $r );
+    return unless $cv->isa( 'B::CV' )
+              and my $gv = $cv->GV
+              ;
+    my $name = '';
+    if ( my $st = $gv->STASH ) {
+        $name = $st->NAME . '::';
+    }
+    my $n = $gv->NAME;
+    if ( $n ) {
+        $name .= $n;
+        if ( $n eq '__ANON__' ) {
+            $name .= ' defined at ' . $gv->FILE . ':' . $gv->LINE;
+        }
+    }
+    return $name;
+}
+
+
+
 # We define posponed/sub as soon as possible to be able watch whole process
 # NOTICE: At this sub we reenter debugger
 sub postponed {
-	if( $options{ trace_load } ) {
-		#TODO: implement local_state to localize debugger state values
-		my $old_inDB =  DB::state( 'inDB' );
-		DB::state( 'inDB', 1 );
-		#FIX: process exceptions
-		mcall( 'trace_load', $DB::dbg, @_ );
+	#TODO: implement local_state to localize debugger state values
+	my $old_inDB =  DB::state( 'inDB' );
+	DB::state( 'inDB', 1 );
+	#FIX: process exceptions
+	emit( 'trace_load', @_ );
 
-		# When we are in debugger and we require module the execution will be
-		# interrupted and we REENTER debugger
-		# TODO: study this case and IT:
-		# T: We are { dd } and run command that 'require'
-		DB::state( 'inDB', $old_inDB );
-	}
+	# When we are in debugger and we require module the execution will be
+	# interrupted and we REENTER debugger
+	# TODO: study this case and IT:
+	# T: We are { dd } and run command that 'require'
+	DB::state( 'inDB', $old_inDB );
 }
 
 
@@ -1109,26 +1122,94 @@ sub postponed {
 our %sig =  (()
 	,trap    =>  \&trap
 	,untrap  =>  \&untrap
-	,call    =>  \&call
-	,uncall  =>  \&uncall
-	,trace   =>  \&trace
-	,untrace =>  \&untrace
-	,stop    =>  \&stop
-	,unstop  =>  \&unstop
-	,frame   =>  \&frame
-	,unframe =>  \&unframe
 );
 
+
+mutate_sub_is_debuggable( \&reg, 0 );
 sub reg {
 	my( $sig, $name, @extra ) =  @_;
 
-	return $DB::sig{ $sig }->( $name, @extra );
+	if( exists $DB::sig{ $sig } ) {
+		return $DB::sig{ $sig }->( $name, @extra );
+	}
+	else {
+		return default_handler( $sig, $name, @extra );
+	}
 }
 
+
+
 sub unreg {
 	my( $sig, $name, @extra ) =  @_;
 
-	return $DB::sig{ "un$sig" }->( $name, @extra );
+	if( exists $DB::sig{ $sig } ) {
+		return $DB::sig{ "un$sig" }->( $name, @extra );
+	}
+	else {
+		return default_unhandler( $sig, $name, @extra );
+	}
+}
+
+
+
+sub emit {
+	my( $name ) =  ( shift );
+
+	print $DB::OUT "Emit event '$name' from ", (caller)[1,2], "\n"   if DB::state( 'ddd' );
+
+	# Get subscribers for the event
+	my $ev; {
+		no strict 'refs';
+		$ev =  defined &{ "${name}_info" }
+			? &{ "${name}_info" }( @_ )
+			: default_handler_info( $name )
+		;
+	}
+
+	my $res =  [];
+	# Events are emitted in context of handler. Event handler should be at least
+	# HASHREF with key 'code' having CODEREF to sub which will process event
+	push @$res, process( $ev->{ $_ }, @_ )   for keys %$ev;
+
+	print $DB::OUT "Event '$name' DONE\n"   if DB::state( 'ddd' );
+
+	return $res;
+}
+
+
+
+sub default_handler_info {
+	return DB::state( "on_$_[0]" ) // {};
+}
+
+
+
+sub default_handler {
+	my( $sig, $name ) =  @_;
+	my $subscribers =  DB::state( "on_$sig" );
+	$subscribers =  DB::state( "on_$sig", {} )   unless $subscribers;
+
+	# HACK: Autovivify subscriber if it does not exists yet
+	# Glory Perl. I love it!
+	return \$subscribers->{ $name };
+}
+
+
+
+sub default_unhandler {
+	my( $sig, $name ) =  @_;
+	my $subscribers =  DB::state( "on_$sig" );
+
+	delete $subscribers->{ $name };
+	DB::state( "on_$sig", undef )   unless keys %$subscribers;
+}
+
+
+
+sub trap_info {
+	my( $file, $line ) =  @_;
+
+	return DB::traps( $file )->{ $line };
 }
 
 
@@ -1170,130 +1251,20 @@ sub untrap {
 
 
 
-sub call {
-	my( $name ) =  @_;
-	my $subscribers =  DB::state( 'on_call' );
-	$subscribers =  DB::state( 'on_call', {} )   unless $subscribers;
-
-	# HACK: Autovivify subscriber if it does not exists yet
-	# Glory Perl. I love it!
-	return \$subscribers->{ $name };
-}
-
-
-
-sub uncall {
-	my( $name ) =  @_;
-	my $subscribers =  DB::state( 'on_call' );
-
-	delete $subscribers->{ $name };
-	DB::state( 'on_call', undef )   unless keys %$subscribers;
-}
-
-
-
-sub trace {
-	my( $name ) =  @_;
-	my $subscribers =  DB::state( 'on_trace' );
-	$subscribers =  DB::state( 'on_trace', {} )   unless $subscribers;
-
-	$DB::trace =  1;
-
-	# HACK: Autovivify subscriber if it does not exists yet
-	# Glory Perl. I love it!
-	return \$subscribers->{ $name };
-}
-
-
-
-sub untrace {
-	my( $name ) =  @_;
-	my $subscribers =  DB::state( 'on_trace' );
-
-	delete $subscribers->{ $name };
-	unless( keys %$subscribers ) {
-		$DB::trace =  0;
-		DB::state( 'on_trace', undef );
-	}
-}
-
-
-
-sub stop {
-	my( $name ) =  @_;
-	my $subscribers =  DB::state( 'on_stop' );
-	$subscribers =  DB::state( 'on_stop', {} )   unless $subscribers;
-
-	# HACK: Autovivify subscriber if it does not exists yet
-	# Glory Perl. I love it!
-	return \$subscribers->{ $name };
-}
-
-
-
-sub unstop {
-	my( $name ) =  @_;
-	my $subscribers =  DB::state( 'on_stop' );
-
-	delete $subscribers->{ $name };
-	DB::state( 'on_stop', undef )   unless keys %$subscribers;
-}
-
-
-
-sub frame {
-	my( $name ) =  @_;
-	my $subscribers =  DB::state( 'on_frame' );
-	$subscribers =  DB::state( 'on_frame', {} )   unless $subscribers;
-
-	# HACK: Autovivify subscriber if it does not exists yet
-	# Glory Perl. I love it!
-	return \$subscribers->{ $name };
-}
-
-
-
-sub unframe {
-	my( $name ) =  @_;
-	my $subscribers =  DB::state( 'on_frame' );
-
-	delete $subscribers->{ $name };
-	DB::state( 'on_frame', undef )   unless keys %$subscribers;
-}
-
-
-
 # TODO: implement: on_enter, on_leave, on_compile
 sub DB_my {
 	&save_context;
 	establish_cleanup \&restore_context;
 
 	my( $p, $f, $l ) =  init();
-	print_state( "DB::DB  ", sprintf "    cursor(DB) => %s, %s, %s\n" ,$p ,$f, $l )   if DB::state( 'ddd' );
-
-	do{ mcall( 'trace_line', $DB::dbg ); }   if $DB::trace;
-
-
-	my $stop =  0;
-	my $traps =  DB::traps();
-	my $trap =  $traps->{ state( 'line' ) };
-
-	for my $key ( keys %$trap ) {
-		next   unless $key =~ /^_/;
-
-		$stop ||=  process( $trap->{ $key } );
-	}
-	#TODO: implement through reduce
-
+	print_state( "DB::DB  ", sprintf "\n    cursor(DB) => %s, %s, %s\n" ,$p ,$f, $l )   if DB::state( 'ddd' );
 
+	emit( 'trace_line' )   if $DB::trace;
 	#TODO: $DB::signal $DB::trace
 
 
-	my $confirm =  1;
-	my $ev =  DB::state( 'on_stop' ) // {};
-	for( keys %$ev ) {
-		$confirm &&=  process( $ev->{ $_ }, $p, $f, $l );
-	}
+	my $stop    =  List::Util::any { $_ } @{ emit( 'trap', $f, $l )     };
+	my $confirm =  List::Util::all { $_ } @{ emit( 'stop', $p, $f, $l ) };
 
 	#IT: stop on breakpoint while stepping
 	return   unless $stop  ||  $DB::single && $confirm  ||  $DB::signal;
@@ -1301,9 +1272,10 @@ sub DB_my {
 
 
 	print_state "\n\nStart to interact with user\n", "\n\n"   if DB::state( 'ddd' );
-	mcall( 'bbreak', $DB::dbg );
-	1 while( defined interact() );
-	mcall( 'abreak', $DB::dbg );
+
+	emit( 'bbreak'   );
+	emit( 'interact' );
+	emit( 'abreak'   );
 }
 
 
@@ -1343,98 +1315,52 @@ sub init {
 
 
 
-# Get a string and process it.
+# When handler returns itself (HASHREF) as result it will keep processing (like 'redo'):
+# specified handler at 'code' key will be reinvoked
+# When handler returns ARRAYREF specified handler also will be reinvoked with
+# evaluated results for each returned item in that array
 sub process {
-	my $str =  shift;
+	my( $handler ) =  @_;
+	my $htype      =  ref $handler;
 
-	my $code =  (ref $str eq 'HASH')
-		? $str->{ code }
-		: $DB::dbg->can( 'process' )
-	;
-
-	#TODO: assert unless $code;
-
-	my @args =  ( $DB::dbg, $str, @_ );
-	PROCESS: {
-		# 0 means : no command found so 'eval( $str )' and keep interaction
-		# TRUE    : command found, keep interaction
-		# HASHREF : eval given { expr } and pass results to { code }
-		# negative: something wrong happened while running the command
-		my $result =  scall( $code, @args );
-		return   unless defined $result;
-
-		if( ref $result eq 'HASH' ) {
-			$code =  $result->{ code };
-			@args =  DB::eval( $result->{ expr } );
-
-			redo PROCESS;
-		}
-		elsif( ref $result eq 'ARRAY' ) {
-			$code =  shift @$result;
+	my( $code, @args );
+	do {
+		if( $htype eq 'ARRAY' ) {
+			print $DB::OUT "Got list of expressions to evaluate in usercontext\n"
+				if DB::state( 'ddd' );
+			$code =  shift @$handler;
 			@args =  ();
-			for my $expr ( @$result ) {
-				#TODO: IT: $expr that is subcommand
-				push @args, [ process( $expr, 1 ) ];
+			for my $expr ( @$handler ) {
+				# $expr should be simple string. If it is not it is special
+				push @args, ref $expr ? process( $expr ) : [ DB::eval( $expr ) ];
+				if( $@ ) {
+					# Pass reference to copy of error message. Value of error
+					# message (global variable) may be changed by anyone
+					$args[-1] =  bless { error => $@ }, 'DB::Error';
+				}
 			}
-
-			redo PROCESS;
+		}
+		elsif( $htype eq 'HASH' ) {
+			$code =  $handler->{ code };
+			@args =  @_;
+		}
+		else {
+			die "Handler type should be ARRAY or HASH";
 		}
 
+		die "Handler is not defined"   unless $code;
 
-		return $result   unless $result == 0  &&  !ref $str;
-	}
+		if( DB::state( 'ddd' ) ) {
+			my $sub =  sub_name( $code ) || $code;
+			my @first =  ref $args[0] eq 'ARRAY' ? @{ $args[0] } : $args[0] // '';
+			print $DB::OUT "Run callback: $sub @first ( @args )\n"
+		}
+	} while(
+		defined( $handler =  scall( $code, @args ) )
+		&&     ( $htype   =  ref $handler          )
+	);
 
-	# else no such command exists the entered string will be evaluated
-	# in __FILE__:__LINE__ context of script we are debugging
-	my @result =  DB::eval( $str );
-	if( $@ ) {
-		print $DB::OUT "ERROR: $@";
-	}
-	elsif( !@_ ) { # Dump results if we do not require them
-		print $DB::OUT "\nEvaluation result:\n"   if DB::state( 'ddd' );
-		@result =  map{ $_ // $DB::options{ undef } } @result;
-		local $" =  $DB::options{ '"' }  //  $";
-		print $DB::OUT "@result\n";
-	}
-
-
-	DB::state( 'db.last_eval', $str );
-	#WARNING: NEVER STORE REFERENCE TO EVALUATION RESULT
-	# This will influence to user's script execution under debugger. Data
-	# will not be DESTROY'ed when it leaves its scope because of we refer it
-
-	# WORKAROUND: https://rt.cpan.org/Public/Bug/Display.html?id=110847
-	# print $DB::OUT "\n";
-
-	return @result   if @_;
-	return 0;
-}
-
-
-
-# TODO: remove clever things out of core. This modules should implement
-# only interface features
-sub interact {
-	return   if @_  &&  $DB::interaction;
-
-	local $DB::interaction =  $DB::interaction +1;
-
-	# local { dd } =  0; # Localization breaks debugger debugging
-	# because it prevents us to turn ON debugging by command: { dd } =  1;
-	my $old =  DB::state( 'dd' );
-	DB::state( 'dd', undef );
-	if( my $str =  mcall( 'interact', $DB::dbg, @_ ) ) {
-		print "\n" .(" "x60 ."*"x20 ."\n")x10   if DB::state( 'ddd' );
-		#NOTICE: we restore { dd } flag before call to &process and not after
-		# as in case of localization
-		DB::state( 'dd', $old );
-		return process( $str );
-	}
-	else {
-		DB::state( 'dd', $old );
-	}
-
-	return;
+	return $handler;
 }
 
 
@@ -1446,11 +1372,12 @@ sub goto {
 	return   unless $options{ trace_goto };
 	return   if DB::state( 'inDB' );
 
+	my $old_inDB =  DB::state( 'inDB' );
 	DB::state( 'inDB', 1 );
 
 	DB::state( 'single', 0 )   if DB::state( 'single' ) & 2;
-	push_frame( 'G' );
-	DB::state( 'inDB', undef )
+	push_frame( my $tmp =  $DB::sub, 'G' );
+	DB::state( 'inDB', $old_inDB )
 };
 
 
@@ -1459,13 +1386,6 @@ sub goto {
 #package DB::Tools;
 # my $x = 0;
 # use Data::Dump qw/ pp /;
-sub test {
-	1;
-	2;
-}
-
-
-
 #Q: Why &DB::sub is called for &pop_frame despite on it is compiled at DB:: package
 # when Scope::Cleanup?
 mutate_sub_is_debuggable( \&pop_frame, 0 );
@@ -1474,89 +1394,12 @@ sub pop_frame {
 	#because this sub is called when flow run out of scope.
 	#TODO: Put this code into eval block
 
+	my $old_inDB =  DB::state( 'inDB' );
 	DB::state( 'inDB', 1 );
-	my $stack =  DB::state( 'stack' );
-	my $last =  pop @$stack;
-	print_state "POP  FRAME <<<< ",
-			"  --  $last->{ sub }\@". @$stack ."\n"
-			."    $last->{ file }:$last->{ line }\n\n"
-		if DB::state( 'ddd' );
-
-	if( @{ DB::state( 'stack' ) } ) {
-		# Restore $DB::single for upper frame
-		DB::state( 'single', DB::state( 'single' ) );
-	} else {
-		# Something nasty happened at &push_frame, because of we are at
-		# &pop_frame already but not "push @{ state( 'stack' ) }" done yet
-		print $DB::OUT "Error happen while &pop_frame. Pay attention to this!\n";
-		$DB::single =  0;
-	}
-	DB::state( 'inDB', undef );
+	emit( 'pop_frame' );
+	DB::state( 'inDB', $old_inDB );
 }
 
-
-
-sub push_frame2 {
-	{ # these lines exists for testing purpose
-		no warnings 'void';
-		test();
-		3;
-	}
-	my $sub =  $DB::sub;
-	print_state "PUSH FRAME $_[0] >>>>  ", "  --  $sub\n"   if DB::state( 'ddd' );
-
-	if( $_[0] ne 'G' ) {
-		# http://stackoverflow.com/questions/34595192/how-to-fix-the-dbgoto-frame
-		# WORKAROUND: for broken frame. Here we are trying to be closer to goto call
-		# Most actual info we get when we trace script step-by-step at this case
-		# those vars have sharp last opcode location.
-		if( !DB::state( 'eval' ) ) {
-			#TODO: If $DB::single == 1 we can skip this because cursor is updated at DB::DB
-			my( $p, $f, $l ) =  caller 2;
-			DB::state( 'package', $p );
-			DB::state( 'file',    $f );
-			DB::state( 'line',    $l );
-			printf $DB::OUT "    cursor(PF) => $p, $f, $l\n"   if DB::state( 'ddd' );
-		}
-
-		my $stack =  DB::state( 'stack' );
-		my $frame =  {()
-			# Until we stop at a callee last known cursor position is the caller position
-			,package     =>  $stack->[-1]{ package }
-			,file        =>  $stack->[-1]{ file    }
-			,line        =>  $stack->[-1]{ line    }
-			,single      =>  $stack->[-1]{ single  }
-			,sub         =>  $sub
-			,goto_frames =>  []
-			,type        =>  $_[0]
-		};
-
-
-		my $ev =  DB::state( 'on_frame' ) // {};
-		for( keys %$ev ) {
-			process( $ev->{ $_ }, $frame );
-		}
-
-		#TODO: Now we push always. Q: How to skip coresopnding &pop_frame?
-		# Think about this feature: if( $confirm ) {
-		push @{ $stack }, $frame;
-	}
-	else {
-		push @{ DB::state( 'goto_frames' ) },
-			[ DB::state( 'package' ), DB::state( 'file' ), DB::state( 'line' ), $sub, $_[0] ]
-	}
-
-
-	my $ev =  DB::state( 'on_call' ) // {};
-	for( keys %$ev ) {
-		process( $ev->{ $_ }, $sub );
-	}
-
-	#TODO: implement functionality using API
-	if( $options{ trace_subs } ) {
-		$DB::dbg->trace_subs();
-	}
-}
 }
 
 
@@ -1564,20 +1407,27 @@ sub push_frame2 {
 sub trace_returns {
 	DB::state( 'inDB', 1 );
 	#FIX: process exceptions
-	mcall( 'trace_returns', $DB::dbg, @_ );
+	emit( 'trace_returns', @_ );
 	DB::state( 'inDB', undef );
 }
 
 
 
 sub push_frame {
-	scall( \&push_frame2, @_ );
+
+	print_state "BEFORE CALL: $_[0]  "   if _ddd;
+	my $old_inDB =  DB::state( 'inDB' );
+	DB::state( 'inDB', 1 );
+
+	emit( 'push_frame', @_ );
 
 	if( DB::state( 'ddd' ) ) {
 		print $DB::OUT "STACK:\n";
 		DB::state( 'stack' );
-		print $DB::OUT "Frame created for $DB::sub\n\n";
 	}
+
+	DB::state( 'inDB', $old_inDB );
+	print_state "BEFORE CALL DONE: $_[0]  "   if _ddd;
 }
 
 
@@ -1585,12 +1435,7 @@ sub push_frame {
 # The sub is installed at compile time as soon as the body has been parsed
 sub sub {
 	#FIX: where to setup 'inDB' state?
-	$DB::_sub =  $DB::sub;
-	print_state "DB::sub  ", "  --  "
-		.sub{
-			(ref $DB::sub ? ref $DB::sub : $DB::sub)
-			."<-- @{[ map{ s#.*?([^/]+)$#$1#; $_ } ((caller 0)[1,2]) ]}\n"
-		}->()
+	print_state "DB::sub  ", "  -->  " .(sub_name( $DB::sub ) // $DB::sub) ."\n"
 		if sub{ DB::state( 'ddd' ) }->() && $DB::sub ne 'DB::can_break';
 		#TODO: We could use { trace_internals } flag to see debugger calls
 
@@ -1600,27 +1445,21 @@ sub sub {
 		# TODO: Here we may log internall subs call chain
 
 		# sub{ DB::state( 'inDB', undef ) }->();
+		#TODO: Do not create extra frames. Speed optimization?
+		# replace return -> goto
 		return &$DB::sub
 	}
 
-	print $DB::OUT "SUB IN: " .@$DB::state ."\n"   if _ddd;
-	sub{ DB::state( 'inDB', 1 ) }->();
-
-
 	# manual localization
-	print $DB::OUT "\nCreating frame for $DB::sub\n"   if DB::state( 'ddd' );
 	establish_cleanup \&DB::pop_frame; # This should be first because we should
 	# start to guard frame before any external call
 
 	#FIX: do not call &pop_frame when &push_frame FAILED
-	push_frame( 'C' );
+	sub{ push_frame( my $tmp =  $DB::sub, 'C' ) }->();
 
-	sub{ DB::state( 'inDB', undef ) }->();
-	print $DB::OUT "SUB OUT: " .@$DB::state ."\n"   if _ddd;
 
 	{
 		BEGIN{ 'strict'->unimport( 'refs' )   if $options{ s } }
-		return &$DB::sub   if !$options{ trace_returns };
 
 
 		if( wantarray ) {                             # list context
@@ -1910,3 +1749,5 @@ POP  FRAME <<<< l:0 b:0:0 e:1 s:1 t:1  --  Apache::DB::handler@1
 }
 
 Maybe because DESTROY is called at first OP after closing block
+
+TODO: Allow to eval: 'shift @_' in debugger
